@@ -13,18 +13,19 @@
 #    * See the License for the specific language governing permissions and
 #    * limitations under the License.
 
-__author__ = 'idanmo'
-
-
 import os
 import tempfile
 import shutil
-import requests
 import tarfile
 import urllib
+import urlparse
+import contextlib
 
 from os.path import expanduser
 
+from cloudify_rest_client import bytes_stream_utils
+from cloudify_rest_client.responses import ListResponse
+
 
 class Blueprint(dict):
 
@@ -45,6 +46,13 @@ class Blueprint(dict):
         """
         return self.get('created_at')
 
+    @property
+    def main_file_name(self):
+        """
+        :return: Blueprint main file name.
+        """
+        return self.get('main_file_name')
+
     @property
     def plan(self):
         """
@@ -54,11 +62,18 @@ class Blueprint(dict):
         """
         return self.get('plan')
 
+    @property
+    def description(self):
+        """
+        Gets the description of the blueprint
+
+        :return: The description of the blueprint.
+        """
+        return self.get('description')
+
 
 class BlueprintsClient(object):
 
-    CONTENT_DISPOSITION_HEADER = 'content-disposition'
-
     def __init__(self, api):
         self.api = api
 
@@ -76,41 +91,69 @@ class BlueprintsClient(object):
                     arcname=os.path.basename(blueprint_directory))
         return tar_path
 
-    def _upload(self, tar_file_obj,
-                application_file_name=None,
-                blueprint_id=None):
+    def _upload(self, archive_location,
+                blueprint_id,
+                application_file_name=None):
         query_params = {}
         if application_file_name is not None:
             query_params['application_file_name'] = \
                 urllib.quote(application_file_name)
 
-        def file_gen():
-            buffer_size = 8192
-            while True:
-                read_bytes = tar_file_obj.read(buffer_size)
-                yield read_bytes
-                if len(read_bytes) < buffer_size:
-                    return
+        uri = '/blueprints/{0}'.format(blueprint_id)
 
-        if blueprint_id is not None:
-            uri = '/blueprints/{0}'.format(blueprint_id)
-            url = '{0}{1}'.format(self.api.url, uri)
-            response = requests.put(url, params=query_params, data=file_gen())
+        # For a Windows path (e.g. "C:\aaa\bbb.zip") scheme is the
+        # drive letter and therefore the 2nd condition is present
+        if urlparse.urlparse(archive_location).scheme and \
+                not os.path.exists(archive_location):
+            # archive location is URL
+            query_params['blueprint_archive_url'] = archive_location
+            data = None
         else:
-            url = '{0}/blueprints'.format(self.api.url)
-            response = requests.post(url, params=query_params, data=file_gen())
-        self.api.verify_response_status(response, 201)
-        return response.json()
+            # archive location is a system path - upload it in chunks
+            data = bytes_stream_utils.request_data_file_stream_gen(
+                archive_location)
 
-    def list(self, _include=None):
+        return self.api.put(uri, params=query_params, data=data,
+                            expected_status_code=201)
+
+    def list(self, _include=None, **kwargs):
         """
         Returns a list of currently stored blueprints.
 
         :param _include: List of fields to include in response.
+        :param kwargs: Optional filter fields. For a list of available fields
+               see the REST service's models.BlueprintState.fields
         :return: Blueprints list.
         """
-        response = self.api.get('/blueprints', _include=_include)
-        return [Blueprint(item) for item in response]
+        response = self.api.get('/blueprints',
+                                _include=_include,
+                                params=kwargs)
+        return ListResponse([Blueprint(item) for item in response['items']],
+                            response['metadata'])
+
+    def publish_archive(self, archive_location, blueprint_id,
+                        blueprint_filename=None):
+        """
+        Publishes a blueprint archive to the Cloudify manager.
+
+        :param archive_location: Path or Url to the archive file.
+        :param blueprint_id: Id of the uploaded blueprint.
+        :param blueprint_filename: The archive's main blueprint yaml filename.
+        :return: Created blueprint.
+
+        Archive file should contain a single directory in which there is a
+        blueprint file named `blueprint_filename` (if `blueprint_filename`
+        is None, this value will be passed to the REST service where a
+        default value should be used).
+        Blueprint ID parameter is available for specifying the
+        blueprint's unique Id.
+        """
+
+        blueprint = self._upload(
+            archive_location,
+            blueprint_id=blueprint_id,
+            application_file_name=blueprint_filename)
+        return Blueprint(blueprint)
 
     def upload(self, blueprint_path, blueprint_id):
         """
@@ -123,21 +166,18 @@ class BlueprintsClient(object):
         Blueprint path should point to the main yaml file of the blueprint
         to be uploaded. Its containing folder will be packed to an archive
         and get uploaded to the manager.
-        An optional blueprint_id parameter is available for specifying the
-        blueprint's unique Id. If not specified, blueprint id will be
-        determined after parsing the blueprint's yaml file.
-
+        Blueprint ID parameter is available for specifying the
+        blueprint's unique Id.
         """
         tempdir = tempfile.mkdtemp()
         try:
             tar_path = self._tar_blueprint(blueprint_path, tempdir)
             application_file = os.path.basename(blueprint_path)
 
-            with open(tar_path, 'rb') as f:
-                blueprint = self._upload(
-                    f,
-                    application_file_name=application_file,
-                    blueprint_id=blueprint_id)
+            blueprint = self._upload(
+                tar_path,
+                blueprint_id=blueprint_id,
+                application_file_name=application_file)
             return Blueprint(blueprint)
         finally:
             shutil.rmtree(tempdir)
@@ -175,27 +215,12 @@ class BlueprintsClient(object):
          (optional)
         :return: The file path of the downloaded blueprint.
         """
-        url = '{0}{1}'.format(self.api.url,
-                              '/blueprints/{0}/archive'.format(blueprint_id))
-        response = requests.get(url, stream=True)
-        self.api.verify_response_status(response, 200)
+        uri = '/blueprints/{0}/archive'.format(blueprint_id)
 
-        if not output_file:
-            if self.CONTENT_DISPOSITION_HEADER not in response.headers:
-                raise RuntimeError(
-                    'Cannot determine attachment filename: {0} header not'
-                    ' found in response headers'.format(
-                        self.CONTENT_DISPOSITION_HEADER))
-            output_file = response.headers[
-                self.CONTENT_DISPOSITION_HEADER].split('filename=')[1]
+        with contextlib.closing(
+                self.api.get(uri, stream=True)) as streamed_response:
 
-        if os.path.exists(output_file):
-            raise OSError("Output file '%s' already exists" % output_file)
+            output_file = bytes_stream_utils.write_response_stream_to_file(
+                streamed_response, output_file)
 
-        with open(output_file, 'wb') as f:
-            for chunk in response.iter_content(chunk_size=8096):
-                if chunk:
-                    f.write(chunk)
-                    f.flush()
-
-        return output_file
+            return output_file
