@@ -13,6 +13,8 @@
 #    * See the License for the specific language governing permissions and
 #    * limitations under the License.
 
+from cloudify_rest_client.responses import ListResponse
+
 
 class Execution(dict):
     """Cloudify workflow execution."""
@@ -70,6 +72,14 @@ class Execution(dict):
         """
         return self.get('parameters')
 
+    @property
+    def is_system_workflow(self):
+        """
+        :return: True if the workflow executed is a system workflow, otherwise
+         False
+        """
+        return self.get('is_system_workflow', False)
+
     @property
     def created_at(self):
         """
@@ -83,19 +93,27 @@ class ExecutionsClient(object):
     def __init__(self, api):
         self.api = api
 
-    def list(self, deployment_id=None, _include=None):
+    def list(self, deployment_id=None, include_system_workflows=False,
+             _include=None, **kwargs):
         """Returns a list of executions.
 
         :param deployment_id: Optional deployment id to get executions for.
+        :param _include_system_workflows: Include executions of system
+               workflows
         :param _include: List of fields to include in response.
+        :param kwargs: Optional filter fields. For a list of available fields
+               see the REST service's models.Execution.fields
         :return: Executions list.
         """
         uri = '/executions'
-        params = None
+        params = {'_include_system_workflows': include_system_workflows}
         if deployment_id:
-            params = {'deployment_id': deployment_id}
+            params['deployment_id'] = deployment_id
+        params.update(kwargs)
+
         response = self.api.get(uri, params=params, _include=_include)
-        return [Execution(item) for item in response]
+        return ListResponse([Execution(item) for item in response['items']],
+                            response['metadata'])
 
     def get(self, execution_id, _include=None):
         """Get execution by its id.
@@ -167,5 +185,5 @@ class ExecutionsClient(object):
         action = 'force-cancel' if force else 'cancel'
         response = self.api.post(uri,
                                  data={'action': action},
-                                 expected_status_code=201)
+                                 expected_status_code=200)
         return Execution(response)
