@@ -52,7 +52,7 @@ class ManagerClient(object):
         Creates context in Cloudify's management machine.
         This method is usually invoked right after management machine
         bootstrap with relevant Cloudify and cloud provider
-         context information.
+        context information.
 
         :param name: Cloud provider name.
         :param context: Context as dict.
@@ -63,3 +63,38 @@ class ManagerClient(object):
                                  data,
                                  expected_status_code=201)
         return response
+
+    def update_context(self, name, context):
+
+        """
+        Updates context in Cloudify's management machine.
+        The context is imperative for the manager to function properly,
+        only use this method if you know exactly what you are doing.
+        Note that if the provider context does not exist, this call will
+        result with an error.
+
+        :param name: Cloud provider name.
+        :param context: Context as dict.
+
+        """
+
+        data = {'name': name, 'context': context}
+        response = self.api.post('/provider/context', data,
+                                 expected_status_code=200,
+                                 params={'update': 'true'})
+        return response
+
+    def set_global_parallel_executions_limit(self, limit):
+        """
+        Sets the global parallel executions limit (relevant only in
+        transient deployment workers mode)
+
+        :param limit: the new (integer) limit for the global parallel
+         executions limit. -1 means no limit.
+        """
+
+        uri = '/provider/context'
+        params = {'global_parallel_executions_limit': limit}
+        response = self.api.patch(uri, data=params,
+                                  expected_status_code=200)
+        return response
