@@ -12,8 +12,9 @@
 #    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #    * See the License for the specific language governing permissions and
 #    * limitations under the License.
+import warnings
 
-__author__ = 'idanmo'
+from cloudify_rest_client.responses import ListResponse
 
 
 class Node(dict):
@@ -87,6 +88,29 @@ class Node(dict):
         return int(self.get(
             'number_of_instances')) if 'number_of_instances' in self else None
 
+    @property
+    def planned_number_of_instances(self):
+        """
+        :return: The planned number of instances this node has.
+        :rtype: int
+        """
+
+        return int(self.get(
+            'planned_number_of_instances')) if 'planned_number_of_instances' \
+                                               in self else None
+
+    @property
+    def deploy_number_of_instances(self):
+        """
+        :return: The number of instances this set for this node when the
+                 deployment was created.
+        :rtype: int
+        """
+
+        return int(self.get(
+            'deploy_number_of_instances')) if 'deploy_number_of_instances' \
+                                              in self else None
+
     @property
     def host_id(self):
         """
@@ -117,14 +141,17 @@ class NodesClient(object):
     def __init__(self, api):
         self.api = api
 
-    def list(self, deployment_id=None, node_id=None, _include=None):
+    def list(self, deployment_id=None, node_id=None, _include=None, **kwargs):
         """
         Returns a list of nodes which belong to the deployment identified
-         by the provided deployment id.
+        by the provided deployment id.
 
         :param deployment_id: The deployment's id to list nodes for.
-        :param node_id: If provided, returns only the requested node.
+        :param node_id: If provided, returns only the requested node. This
+                        parameter is deprecated, use 'id' instead.
         :param _include: List of fields to include in response.
+        :param kwargs: Optional filter fields. for a list of available fields
+               see the REST service's models.DeploymentNode.fields
         :return: Nodes.
         :rtype: list
         """
@@ -132,16 +159,20 @@ class NodesClient(object):
         if deployment_id:
             params['deployment_id'] = deployment_id
         if node_id:
-            params['node_id'] = node_id
+            warnings.warn("'node_id' filtering capability is deprecated, use"
+                          " 'id' instead", DeprecationWarning)
+            params['id'] = node_id
+        params.update(kwargs)
         if not params:
             params = None
         response = self.api.get('/nodes', params=params, _include=_include)
-        return [Node(item) for item in response]
+        return ListResponse([Node(item) for item in response['items']],
+                            response['metadata'])
 
     def get(self, deployment_id, node_id, _include=None):
         """
         Returns the node which belongs to the deployment identified
-         by the provided deployment id .
+        by the provided deployment id .
 
         :param deployment_id: The deployment's id of the node.
         :param node_id: The node id.
